#include <nbtk/nbtk.h>

#include "hrn-queue-item.h"
#include "hrn-queue-item-factory.h"

struct _HrnQueueItemFactoryPrivate {
    int dummy;
};

enum {
    ITEM_CLICKED,
    LAST_SIGNAL
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), HRN_TYPE_QUEUE_ITEM_FACTORY, HrnQueueItemFactoryPrivate))
static guint32 signals[LAST_SIGNAL] = {0, };

static void item_factory_init (NbtkItemFactoryIface *iface);
G_DEFINE_TYPE_WITH_CODE (HrnQueueItemFactory, hrn_queue_item_factory, G_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE (NBTK_TYPE_ITEM_FACTORY,
                                                item_factory_init));

static void
hrn_queue_item_factory_finalize (GObject *object)
{
    G_OBJECT_CLASS (hrn_queue_item_factory_parent_class)->finalize (object);
}

static void
hrn_queue_item_factory_dispose (GObject *object)
{
    G_OBJECT_CLASS (hrn_queue_item_factory_parent_class)->dispose (object);
}

static void
hrn_queue_item_factory_set_property (GObject      *object,
                                     guint         prop_id,
                                     const GValue *value,
                                     GParamSpec   *pspec)
{
    switch (prop_id) {

    default:
        break;
    }
}

static void
hrn_queue_item_factory_get_property (GObject    *object,
                                     guint       prop_id,
                                     GValue     *value,
                                     GParamSpec *pspec)
{
    switch (prop_id) {

    default:
        break;
    }
}

static void
hrn_queue_item_factory_class_init (HrnQueueItemFactoryClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;

    o_class->dispose = hrn_queue_item_factory_dispose;
    o_class->finalize = hrn_queue_item_factory_finalize;
    o_class->set_property = hrn_queue_item_factory_set_property;
    o_class->get_property = hrn_queue_item_factory_get_property;

    g_type_class_add_private (klass, sizeof (HrnQueueItemFactoryPrivate));

    signals[ITEM_CLICKED] = g_signal_new ("item-clicked",
                                          G_TYPE_FROM_CLASS (klass),
                                          G_SIGNAL_RUN_LAST |
                                          G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                          g_cclosure_marshal_VOID__OBJECT,
                                          G_TYPE_NONE, 1,
                                          HRN_TYPE_QUEUE_ITEM);
}

static gboolean
item_clicked_cb (ClutterActor        *actor,
                 ClutterButtonEvent  *event,
                 HrnQueueItemFactory *factory)
{
    if (event->button != 1) {
        return FALSE;
    }

    if (event->click_count == 2) {
        g_signal_emit (factory, signals[ITEM_CLICKED], 0, actor);
        return TRUE;
    }

    return FALSE;
}

static ClutterActor *
create_item (NbtkItemFactory *factory)
{
    HrnQueueItem *item;

    item = g_object_new (HRN_TYPE_QUEUE_ITEM, NULL);
    clutter_actor_set_reactive ((ClutterActor *) item, TRUE);
    g_signal_connect (item, "button-release-event",
                      G_CALLBACK (item_clicked_cb), factory);

    return (ClutterActor *) item;
}

static void
item_factory_init (NbtkItemFactoryIface *iface)
{
    iface->create = create_item;
}

static void
hrn_queue_item_factory_init (HrnQueueItemFactory *self)
{
    self->priv = GET_PRIVATE (self);
}

HrnQueueItemFactory *
hrn_queue_item_factory_new (void)
{
    HrnQueueItemFactory *factory;

    factory = g_object_new (HRN_TYPE_QUEUE_ITEM_FACTORY, NULL);
    return factory;
}
