#include <glib/gi18n.h>

#include <bickley/bkl-item.h>
#include <bickley/bkl-item-audio.h>

#include "hrn-queue-item.h"

enum {
    PROP_0,
    PROP_ITEM,
    PROP_URI
};

enum {
    LAST_SIGNAL,
};

struct _HrnQueueItemPrivate {
    NbtkWidget *primary;
    NbtkWidget *secondary;

    BklItem *item;
    char *uri;
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), HRN_TYPE_QUEUE_ITEM, HrnQueueItemPrivate))
G_DEFINE_TYPE (HrnQueueItem, hrn_queue_item, NBTK_TYPE_TABLE);
static guint32 signals[LAST_SIGNAL] = {0,};

static void
hrn_queue_item_finalize (GObject *object)
{
    HrnQueueItem *self = (HrnQueueItem *) object;

    G_OBJECT_CLASS (hrn_queue_item_parent_class)->finalize (object);
}

static void
hrn_queue_item_dispose (GObject *object)
{
    HrnQueueItem *self = (HrnQueueItem *) object;

    G_OBJECT_CLASS (hrn_queue_item_parent_class)->dispose (object);
}

static void
update_tile (HrnQueueItem *item)
{
    HrnQueueItemPrivate *priv = item->priv;
    GPtrArray *artists;
    const char *artist = NULL, *title = NULL;

    title = bkl_item_audio_get_title ((BklItemAudio *) priv->item);
    artists = bkl_item_audio_get_artists ((BklItemAudio *) priv->item);
    if (artists) {
        /* FIXME: Handle multiple artists (again) */
        artist = artists->pdata[0];
    }

    nbtk_label_set_text ((NbtkLabel *) priv->primary,
                         title ? title : _("Unknown Title"));
    nbtk_label_set_text ((NbtkLabel *) priv->secondary,
                         artist ? artist : _("Unknown Artist"));
}

static void
hrn_queue_item_set_property (GObject      *object,
                          guint         prop_id,
                          const GValue *value,
                          GParamSpec   *pspec)
{
    HrnQueueItem *self = (HrnQueueItem *) object;
    HrnQueueItemPrivate *priv = self->priv;
    GObject *obj;

    switch (prop_id) {
    case PROP_ITEM:
        obj = g_value_get_object (value);
        if (obj != priv->item) {
            if (priv->item) {
                g_object_unref (priv->item);
            }
            priv->item = g_object_ref (obj);

            update_tile (self);
        }
        break;

    case PROP_URI:
        if (priv->uri) {
            g_free (priv->uri);
        }
        priv->uri = g_value_dup_string (value);
        if (priv->item == NULL) {
            char *filename, *unesc;

            filename = g_path_get_basename (priv->uri);
            unesc = g_uri_unescape_string (filename, NULL);
            g_free (filename);

            nbtk_label_set_text ((NbtkLabel *) priv->primary, unesc);
            nbtk_label_set_text ((NbtkLabel *) priv->secondary, "");

            g_free (unesc);
        }
        break;

    default:
        break;
    }
}

static void
hrn_queue_item_get_property (GObject    *object,
                             guint       prop_id,
                             GValue     *value,
                             GParamSpec *pspec)
{
    HrnQueueItem *self = (HrnQueueItem *) object;
    HrnQueueItemPrivate *priv = self->priv;

    switch (prop_id) {
    case PROP_ITEM:
        g_value_set_object (value, priv->item);
        break;

    case PROP_URI:
        g_value_set_string (value, priv->uri);
        break;

    default:
        break;
    }
}

static void
hrn_queue_item_class_init (HrnQueueItemClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;

    o_class->dispose = hrn_queue_item_dispose;
    o_class->finalize = hrn_queue_item_finalize;
    o_class->set_property = hrn_queue_item_set_property;
    o_class->get_property = hrn_queue_item_get_property;

    g_type_class_add_private (klass, sizeof (HrnQueueItemPrivate));

    g_object_class_install_property (o_class, PROP_ITEM,
                                     g_param_spec_object ("item", "", "",
                                                          BKL_TYPE_ITEM,
                                                          G_PARAM_STATIC_STRINGS |
                                                          G_PARAM_READWRITE));
    g_object_class_install_property (o_class, PROP_URI,
                                     g_param_spec_string ("uri", "", "", "",
                                                          G_PARAM_STATIC_STRINGS |
                                                          G_PARAM_READWRITE));
}

static void
hrn_queue_item_init (HrnQueueItem *self)
{
    HrnQueueItemPrivate *priv = GET_PRIVATE (self);

    self->priv = priv;

    priv->primary = nbtk_label_new ("");
    nbtk_widget_set_style_class_name (priv->primary, "HrnQueueItemLabel");
    nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
                                          (ClutterActor *) priv->primary, 0, 0,
                                          "x-expand", FALSE,
                                          "x-fill", FALSE,
                                          "y-expand", FALSE,
                                          "y-fill", FALSE,
                                          "x-align", 0.0,
                                          NULL);

    priv->secondary = nbtk_label_new ("");
    nbtk_widget_set_style_class_name (priv->secondary, "HrnQueueItemMetaLabel");
    nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
                                          (ClutterActor *) priv->secondary, 1, 0,
                                          "x-expand", FALSE,
                                          "x-fill", FALSE,
                                          "y-expand", FALSE,
                                          "y-fill", FALSE,
                                          "x-align", 0.0,
                                          NULL);
}

